'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.generateModelAst = generateModelAst;
exports.default = generateModel;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _recast = require('recast');

var _read = require('../read');

var _capitalization = require('../util/capitalization');

var _generatePerField = require('../util/generatePerField');

var _generatePerField2 = _interopRequireDefault(_generatePerField);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function read(name) {
  return _fs2.default.readFileSync(__dirname + '/templates/' + name + '.js.template', 'utf8');
}

var templates = {
  base: read('base'),
  singularAssociation: read('singularAssociation'),
  paginatedAssociation: read('paginatedAssociation')
};

function buildAst(template, _ref) {
  var typeName = _ref.typeName,
      fieldName = _ref.fieldName,
      argsStr = _ref.argsStr,
      ReturnTypeName = _ref.ReturnTypeName,
      query = _ref.query;

  var argsWithDefaultsStr = argsStr.replace('lastCreatedAt', 'lastCreatedAt = 0').replace('limit', 'limit = 10');
  return (0, _read.templateToAst)(template, {
    typeName: typeName,
    fieldName: fieldName,
    argsStr: argsWithDefaultsStr,
    ReturnTypeName: ReturnTypeName,
    query: query
  });
}

var generators = {
  base: function base(_ref2) {
    var typeName = _ref2.typeName,
        TypeName = _ref2.TypeName;

    return (0, _read.templateToAst)(templates.base, { typeName: typeName, TypeName: TypeName });
  },
  belongsTo: function belongsTo(replacements) {
    return buildAst(templates.singularAssociation, replacements);
  },
  belongsToMany: function belongsToMany(replacements) {
    var typeName = replacements.typeName,
        fieldName = replacements.fieldName;

    return buildAst(templates.paginatedAssociation, _extends({}, replacements, {
      query: '_id: { $in: ' + typeName + '.' + fieldName + 'Ids || [] }'
    }));
  },

  // TODO: write test and implement
  // hasOne({ typeName, fieldName, ReturnTypeName }, { as }) {
  //   return templateToAst(templates.singularAssociation, {
  //     typeName,
  //     fieldName,
  //     ReturnTypeName,
  //   });
  // },
  hasMany: function hasMany(replacements, _ref3) {
    var as = _ref3.as;
    var typeName = replacements.typeName;

    return buildAst(templates.paginatedAssociation, _extends({}, replacements, {
      query: (as || typeName) + 'Id: ' + typeName + '._id'
    }));
  },
  hasAndBelongsToMany: function hasAndBelongsToMany(replacements, _ref4) {
    var as = _ref4.as;
    var typeName = replacements.typeName;

    return buildAst(templates.paginatedAssociation, _extends({}, replacements, {
      query: (as || typeName) + 'Ids: ' + typeName + '._id'
    }));
  }
};

function generateModelAst(inputSchema) {
  var type = inputSchema.definitions[0];
  var TypeName = type.name.value;
  var typeName = (0, _capitalization.lcFirst)(TypeName);

  var ast = generators.base({ TypeName: TypeName, typeName: typeName });

  // XXX: rather than hardcoding in array indices it would be less brittle to
  // walk the tree using https://github.com/benjamn/ast-types
  var classMethodsAst = ast.program.body[2] // export
  .declaration // class declaration
  .body.body;

  var findOneMethod = classMethodsAst.find(function (m) {
    return m.key.name === 'all';
  });
  var nextIndex = classMethodsAst.indexOf(findOneMethod) + 1;

  (0, _generatePerField2.default)(type, generators).forEach(function (resolverFunctionAst) {
    var classMethodAst = resolverFunctionAst.program.body[0] // class declaration
    .body.body[0]; // classMethod

    classMethodsAst.splice(nextIndex, 0, classMethodAst);
    nextIndex += 1;
  });

  return ast;
}

function generateModel(inputSchema) {
  var ast = generateModelAst(inputSchema);
  return (0, _recast.print)(ast, { trailingComma: true }).code;
}