'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateResolversAst = generateResolversAst;
exports.default = generateResolvers;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _recast = require('recast');

var _read = require('../read');

var _generatePerField = require('../util/generatePerField');

var _generatePerField2 = _interopRequireDefault(_generatePerField);

var _capitalization = require('../util/capitalization');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function read(name) {
  return _fs2.default.readFileSync(__dirname + '/templates/' + name + '.js.template', 'utf8');
}

var templates = {
  base: read('base'),
  fieldOfType: read('fieldOfType'),
  paginatedField: read('paginatedField')
};

function generateResolver(template) {
  return function (_ref) {
    var TypeName = _ref.TypeName,
        typeName = _ref.typeName,
        fieldName = _ref.fieldName,
        argsStr = _ref.argsStr;

    return (0, _read.templateToAst)(template, { typeName: typeName, TypeName: TypeName, fieldName: fieldName, argsStr: argsStr });
  };
}

var generators = {
  base: function base(_ref2) {
    var typeName = _ref2.typeName,
        TypeName = _ref2.TypeName;

    return (0, _read.templateToAst)(templates.base, { typeName: typeName, TypeName: TypeName });
  },

  belongsTo: generateResolver(templates.fieldOfType),
  belongsToMany: generateResolver(templates.paginatedField),
  hasOne: generateResolver(templates.fieldOfType),
  hasMany: generateResolver(templates.paginatedField),
  hasAndBelongsToMany: generateResolver(templates.paginatedField)
};

function generateResolversAst(inputSchema) {
  var type = inputSchema.definitions[0];
  var TypeName = type.name.value;
  var typeName = (0, _capitalization.lcFirst)(TypeName);

  var ast = generators.base({ TypeName: TypeName, typeName: typeName });

  // XXX: rather than hardcoding in array indices it would be less brittle to
  // walk the tree using https://github.com/benjamn/ast-types
  var typeResolversAst = ast.program.body[0] // const
  .declarations[0].init // object expression
  .properties[0].value; // object value

  (0, _generatePerField2.default)(type, generators).forEach(function (resolverFunctionAst) {
    var resolverProperty = resolverFunctionAst.program.body[0] // variable declaration
    .declarations[0].init // object expression
    .properties[0];

    typeResolversAst.properties.push(resolverProperty);
  });

  return ast;
}

function generateResolvers(inputSchema) {
  var ast = generateResolversAst(inputSchema);
  return (0, _recast.print)(ast, { trailingComma: true }).code;
}