'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateSchema;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _lodash = require('lodash.clonedeep');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.includes');

var _lodash4 = _interopRequireDefault(_lodash3);

var _graphql = require('../util/graphql');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-param-reassign */

function generateSchema(inputSchema) {
  // Check that the input looks like we expect -- a single ObjectType definition
  (0, _assert2.default)(inputSchema.kind === 'Document');
  (0, _assert2.default)(inputSchema.definitions.length === 1);

  var outputSchema = (0, _lodash2.default)(inputSchema);

  var type = outputSchema.definitions[0];
  var typeName = type.name.value;

  var createInputFields = [];
  var updateInputFields = [];
  type.fields.forEach(function (field) {
    var directivesByName = {};
    field.directives.forEach(function (directive) {
      directivesByName[directive.name.value] = directive;
      (0, _graphql.applyCustomDirectives)(field);
    });

    // XXX: Not sure if this the correct logic but it makes my tests pass
    // TODO: check for @unmodifiable
    var possibleInputType = field.type;
    var inputTypeModifier = '';
    if (possibleInputType.kind === 'NonNullType') {
      possibleInputType = possibleInputType.type;
      inputTypeModifier = '!';
    }

    if (possibleInputType.kind === 'NamedType') {
      var isScalarField = (0, _lodash4.default)(_graphql.SCALAR_TYPE_NAMES, possibleInputType.name.value);
      var inputField = void 0;
      if (isScalarField || !!directivesByName.enum) {
        inputField = field;
      } else {
        inputField = (0, _graphql.buildField)(field.name.value + 'Id', [], 'ObjID' + inputTypeModifier);
      }

      createInputFields.push(inputField);
      if (!directivesByName.unmodifiable) {
        updateInputFields.push(inputField);
      }
    }

    field.directives = [];
  });

  type.fields.unshift((0, _graphql.buildField)('id', [], 'ObjID!'));
  type.fields.push((0, _graphql.buildField)('createdAt', [], 'Float!'));
  type.fields.push((0, _graphql.buildField)('updatedAt', [], 'Float!'));

  var queryOneField = (0, _graphql.buildField)(typeName.toLowerCase(), [(0, _graphql.idArgument)()], typeName);
  var queryAllField = (0, _graphql.buildField)(typeName.toLowerCase() + 's', [], '[' + typeName + '!]');
  (0, _graphql.addPaginationArguments)(queryAllField);
  outputSchema.definitions.push((0, _graphql.buildTypeExtension)((0, _graphql.buildTypeDefinition)('Query', [queryAllField, queryOneField])));

  var createInputTypeName = 'Create' + typeName + 'Input';
  outputSchema.definitions.push((0, _graphql.buildTypeDefinition)(createInputTypeName, createInputFields, 'InputObjectTypeDefinition'));

  var updateInputTypeName = 'Update' + typeName + 'Input';
  outputSchema.definitions.push((0, _graphql.buildTypeDefinition)(updateInputTypeName, updateInputFields, 'InputObjectTypeDefinition'));

  // Create update input type if readonly fields

  outputSchema.definitions.push((0, _graphql.buildTypeExtension)((0, _graphql.buildTypeDefinition)('Mutation', [(0, _graphql.buildField)('create' + typeName, [(0, _graphql.buildArgument)('input', createInputTypeName + '!')], typeName), (0, _graphql.buildField)('update' + typeName, [(0, _graphql.idArgument)(), (0, _graphql.buildArgument)('input', updateInputTypeName + '!')], typeName), (0, _graphql.buildField)('remove' + typeName, [(0, _graphql.idArgument)()], 'Boolean')])));

  outputSchema.definitions.push((0, _graphql.buildTypeExtension)((0, _graphql.buildTypeDefinition)('Subscription', [(0, _graphql.buildField)(typeName.toLowerCase() + 'Created', [], typeName), (0, _graphql.buildField)(typeName.toLowerCase() + 'Updated', [], typeName), (0, _graphql.buildField)(typeName.toLowerCase() + 'Removed', [], 'ObjID')])));

  return outputSchema;
}